class ArtistsController < ApplicationController
  layout 'test' if MagicNumber['test_layout']
  if MagicNumber['run_mode'] == 0
    before_filter :authenticate_user, :only => [:new, :create, :edit, :update, :destroy]
    before_filter :authenticate_author, :only => [:new, :create, :edit, :update, :destroy]
  else
    before_filter :authenticate_resource_reader, :only => [:index, :show, :resource_pictures]
    before_filter :authenticate_user, :only => [:new, :create, :edit, :update, :destroy]
    before_filter :authenticate_author, :only => [:new, :create, :edit, :update, :destroy]
  end
  before_filter :authenticate_admin!, :only => [:list, :browse]

  def index
    @page = Artist.page params[:page]
    @page_size = Artist.page_size params[:page_size]
    @artists = Artist.list(@page, @page_size)

    respond_to do |format|
      format.html {
        @paginate = Artist.list_paginate(@page, @page_size)
      }
      format.json { render :json => @artists.to_json(Artist.list_json_opt) }
    end
  end

  def show
    @ar = Artist.show(params[:id], [@user, @admin, @demand_user])

    respond_to do |format|
      format.html # show.html.erb
      format.json { render :json => @ar.to_json(Artist.show_json_opt) }
    end
  end

  def resource_pictures
    @ar = Artist.show(params[:id], [@user, @admin, @demand_user])
    @page = Author.page params[:page]
    @page_size = Author.resource_picture_page_size params[:page_size]
    @resource_pictures = ResourcePicture.himlist(@ar, @page, @page_size)
    respond_to do |format|
      format.html {
        @paginate = ResourcePicture.himlist_paginate(@ar, @page, @page_size)
      }
      format.json { render json: @resource_pictures.to_json(ResourcePicture.list_json_opt) }
    end
  end
  
  def count
    @ar = {:count => Artist.visible_count}
    respond_to do |format|
      format.json { render json: @ar.to_json }
    end
  end
  
  def list
    @artists = Artist.all

    respond_to do |format|
      format.html { render layout: 'system' }
      format.json { render json: @artists }
    end
  end

  def browse
    @ar = Artist.find(params[:id])

    respond_to do |format|
      format.html { render layout: 'system' }
      format.json { render json: @ar }
    end
  end

  # GET /artists/new
  # GET /artists/new.json
  def new
    @ar = Artist.new
    @ar.supply_default 

    respond_to do |format|
      format.html # new.html.erb
      format.js
      format.json { render json: @ar.to_json(Artist.show_json_opt) }
    end
  end

  # GET /artists/1/edit
  def edit
    @ar = Artist.edit(params[:id], @author)
    respond_to do |format|
      format.html 
      format.js
    end
  end

  # POST /artists
  # POST /artists.json
  def create
    @ar = Artist.new()
    @ar.supply_default 
    @ar.attributes = params[:artist]
    @ar.overwrite @author
    respond_to do |format|
      if @ar.save
        flash[:notice] = I18n.t('flash.notice.created', :model => Artist.model_name.human)
        format.html { redirect_to root_path }
        format.json { render json: @ar.to_json(Artist.show_json_opt), status: :created, location: @artist }
      else
        flash[:notice] = I18n.t('flash.notice.not_created', :model => Artist.model_name.human)
        format.html { render action: "new" }
        format.json { render json: @ar.errors, status: :unprocessable_entity }
      end
    end
  end

  # PUT /artists/1
  # PUT /artists/1.json
  def update
    @ar = Artist.edit(params[:id], @author)
    @ar.attributes = params[:artist]
    @ar.overwrite @author

    respond_to do |format|
      if @ar.save
        flash[:notice] = I18n.t('flash.notice.updated', :model => Artist.model_name.human)
        format.html { redirect_to '/home/configure' }
        format.json { head :ok }
      else
        flash[:notice] = I18n.t('flash.notice.not_updated', :model => Artist.model_name.human)
        format.html { render action: "edit" }
        format.json { render json: @ar.errors, status: :unprocessable_entity }
      end
    end
  end

  # DELETE /artists/1
  # DELETE /artists/1.json
  def destroy
    @ar = Artist.edit(params[:id], @author)
    @ar.destroy

    respond_to do |format|
      format.html { redirect_to artists_url }
      format.json { head :ok }
    end
  end
end
