class PanelsController < ApplicationController
  layout 'test' if MagicNumber['test_layout']
  if MagicNumber['run_mode'] == 0
    before_filter :authenticate_user, :only => [:new, :edit, :create, :update, :destroy, :catch]
    before_filter :authenticate_author, :only => [:new, :edit, :create, :update, :destroy, :catch]
  else
    before_filter :authenticate_reader, :only => [:index, :show]
    before_filter :authenticate_user, :only => [:new, :edit, :create, :update, :destroy, :catch]
    before_filter :authenticate_author, :only => [:new, :edit, :create, :update, :destroy, :catch]
  end
  before_filter :authenticate_admin!, :only => [:list, :browse]

  public
  
  def index
    @page = Panel.page params[:page]
    @page_size = Panel.page_size params[:page_size]
    @panels = Panel.list(@page, @page_size)

    respond_to do |format|
      format.html {
        @paginate = Panel.list_paginate(@page, @page_size)
      }
      format.json { render text: Panel.list_as_json_text(@panels) }
      format.atom 
      format.rss { render :layout => false }
    end
  end

  def show
    @panel = Panel.show(params[:id], [@user, @admin])

    respond_to do |format|
      format.html {
        if @author
          @new_scrolls = Scroll.mylist(@author, 1, 5)
          @fresh_scrolls = ScrollPanel.mylist(@author, 1, 5).map {|sp| sp.scroll}
          @new_sheets = Sheet.mylist(@author, 1, 5)
          @fresh_sheets = SheetPanel.mylist(@author, 1, 5).map {|sp| sp.sheet}
        end
      }
      format.json { render json: @panel.panel_elements_as_json }
      format.jsonp {
        render :json => "callback(" + @panel.panel_elements_as_json + ");"
      }
    end
  end

  def count
    @panel = {:count => Panel.visible_count}
    respond_to do |format|
      format.json { render json: @panel.to_json }
    end
  end
  
  def list
    @panels = Panel.all :order => 'updated_at'

    respond_to do |format|
      format.html { render layout: 'system' }
      format.json { render json: @panels }
    end
  end

  def browse
    @panel = Panel.find(params[:id])

    respond_to do |format|
      format.html { render layout: 'system' }
    end
  end

  def new
    @panel = Panel.new
    @panel.supply_default

    respond_to do |format|
      format.html
      format.js
      format.json { render json: @panel.panel_elements_as_json }
    end
  end

  def edit
    @panel = Panel.edit(params[:id], @author)
    respond_to do |format|
      format.html
      format.js
    end
  end

  def create
    @panel = Panel.new
    @panel.supply_default
    jsn = nil
    if params[:json]
      jsn = JSON.parse_no_except(params[:json])
    end
    @prm = params[:panel] || jsn
    respond_to do |format|
      if @panel.store @prm, @author
        flash[:notice] = I18n.t('flash.notice.created', :model => Panel.model_name.human)
        format.html { redirect_to @panel }
        format.json { render json: @panel.panel_elements_as_json, status: :created, location: @panel }
      else
        format.html { render action: "new" }
        format.json { render json: @panel.errors, status: :unprocessable_entity }
      end
    end
  end

  def update
    @panel = Panel.edit(params[:id], @author)
    jsn = nil
    if params[:json]
      jsn = JSON.parse(params[:json])
    end
    @prm = params[:panel] || jsn
    respond_to do |format|
      if @panel.store @prm, @author
        flash[:notice] = I18n.t('flash.notice.updated', :model => Panel.model_name.human)
        format.html { redirect_to @panel }
        format.json { head :ok }
      else
        format.html { render action: "edit" }
        format.json { render json: @panel.errors, status: :unprocessable_entity }
      end
    end
  end

  def destroy
    @panel = Panel.edit(params[:id], @author)
    respond_to do |format|
      Panel.transaction do
        if @panel.destroy_with_elements
          flash[:notice] = I18n.t('flash.notice.destroyed', :model => Panel.model_name.human)
          format.html { redirect_to '/home/panels' }
          format.json { head :ok }
        else
          flash[:notice] = I18n.t('flash.notice.not_destroyed', :model => Panel.model_name.human)
          format.html { redirect_to @panel }
          format.json { render json: @panel.errors, status: :unprocessable_entity }
        end
      end
    end
  end
  
  def catch
    @panel = Panel.edit(params[:id], @author)

    respond_to do |format|
      @author.working_panel_id = @panel.id
      if @author.save
        format.html { redirect_to @panel }
      else
      end
    end
  end

end
