class SheetPanelsController < ApplicationController
  layout 'test' if MagicNumber['test_layout']
  if MagicNumber['run_mode'] == 0
    before_filter :authenticate_user, :only => [:new, :create, :edit, :update, :destroy]
    before_filter :authenticate_author, :only => [:new, :create, :edit, :update, :destroy]
  else
    before_filter :authenticate_reader, :only => [:index, :show]
    before_filter :authenticate_user, :only => [:new, :create, :edit, :update, :destroy]
    before_filter :authenticate_author, :only => [:new, :create, :edit, :update, :destroy]
  end
  before_filter :authenticate_admin!, :only => [:list, :browse]

  def index
    @page = SheetPanel.page params[:page]
    @page_size = SheetPanel.page_size params[:page_size]
    @sheet_panels = SheetPanel.list(@page, @page_size)

    respond_to do |format|
      format.html {
        @paginate = SheetPanel.list_paginate(@page, @page_size)
      }
      format.json { render :json => @sheet_panels.to_json(SheetPanel.list_json_opt) }
    end
  end

  def show
    @sheet_panel = SheetPanel.show(params[:id], [@user, @admin])

    respond_to do |format|
      format.html # show.html.erb
      format.json { render json: @sheet_panel.sheet_panel_as_json(@author) }
    end
  end
  
  def list
    @sheet_panels = SheetPanel.all

    respond_to do |format|
      format.html { render layout: 'system' }# index.html.erb
      format.json { render json: @sheet_panels }
    end
  end

  def browse
    @sheet_panel = SheetPanel.find(params[:id])

    respond_to do |format|
      format.html { render layout: 'system' } # show.html.erb
      format.json { render json: @sheet_panel }
    end
  end
  
  def new
    @sheet_panel = SheetPanel.new 
    @sheet_panel.supply_default
    respond_to do |format|
      format.html # new.html.erb
      format.js
      format.json { render json: @sheet_panel.sheet_panel_as_json(@author) }
    end
  end

  def edit
    @sheet_panel = SheetPanel.edit(params[:id], @author)
    respond_to do |format|
      format.html 
      format.js
    end
  end

  def create
    @sheet_panel = SheetPanel.new 
    @sheet_panel.supply_default
    @sheet_panel.attributes = params[:sheet_panel]
    @sheet_panel.overwrite @author
    @sheet = Sheet.edit(@sheet_panel.sheet_id, @author) if @sheet_panel.sheet_id
    @panel = Panel.show(@sheet_panel.panel_id, @author) if @sheet_panel.panel_id
    
    respond_to do |format|
      if @sheet_panel.store
        flash[:notice] = I18n.t('flash.notice.created', :model => SheetPanel.model_name.human)
        format.html { redirect_to play_sheet_path(@sheet) }
        format.json { render json: @sheet_panel.sheet_panel_as_json(@author) }
      else
        flash[:notice] = I18n.t('flash.notice.not_created', :model => SheetPanel.model_name.human)
        format.html { render action: "new" }
        format.json { render json: @sheet_panel.errors, status: :unprocessable_entity }
      end
    end
  end
  
  def update
    @sheet_panel = SheetPanel.edit(params[:id], @author)
    ot = @sheet_panel.t
    @sheet_panel.attributes = params[:sheet_panel]
    @sheet_panel.overwrite @author
    @sheet = Sheet.edit(@sheet_panel.sheet_id, @author) if @sheet_panel.sheet_id
    respond_to do |format|
      if @sheet_panel.store ot
        flash[:notice] = I18n.t('flash.notice.updated', :model => SheetPanel.model_name.human)
        format.html { redirect_to play_sheet_path(@sheet) }
        format.json { head :ok }
      else
        flash[:notice] = I18n.t('flash.notice.not_updated', :model => SheetPanel.model_name.human)
        format.html { render action: "edit" }
        format.json { render json: @sheet_panel.errors, status: :unprocessable_entity }
      end
    end
  end

  def destroy
    @sheet_panel = SheetPanel.edit(params[:id], @author)
    @sheet = Sheet.edit(@sheet_panel.sheet_id, @author) if @sheet_panel.sheet_id
    respond_to do |format|
      if @sheet_panel.destroy_and_shorten
        flash[:notice] = I18n.t('flash.notice.destroyed', :model => SheetPanel.model_name.human)
        format.html { redirect_to play_sheet_path(@sheet) }
        format.json { head :ok }
      else
        flash[:notice] = I18n.t('flash.notice.not_destroyed', :model => SheetPanel.model_name.human)
        format.html { redirect_to sheet_panel_path(@sheet_panel) }
        format.json { render json: @sheet_panel.errors, status: :unprocessable_entity }
      end
    end
  end
end
