class SheetsController < ApplicationController
  layout 'test' if MagicNumber['test_layout']
  if MagicNumber['run_mode'] == 0
    before_filter :authenticate_user, :only => [:new, :create, :edit, :update, :destroy]
    before_filter :authenticate_author, :only => [:new, :create, :edit, :update, :destroy]
  else
    before_filter :authenticate_reader, :only => [:index, :show, :play]
    before_filter :authenticate_user, :only => [:new, :create, :edit, :update, :destroy]
    before_filter :authenticate_author, :only => [:new, :create, :edit, :update, :destroy]
  end
  before_filter :authenticate_admin!, :only => [:list, :browse]

  def index
    @page = Sheet.page params[:page]
    @page_size = Sheet.page_size params[:page_size]
    @sheets = Sheet.list(@page, @page_size)
    respond_to do |format|
      format.html {
        @paginate = Sheet.list_paginate(@page, @page_size)
      }
      format.json { render json: @sheets.to_json(Sheet.list_json_opt) }
      format.atom 
      format.rss
    end
  end

  def show
    @sheet = Sheet.show(params[:id], [@user, @admin])

    respond_to do |format|
      format.html {
        if @author
          @new_panels = Panel.mylist(@author, 1, 5)
          @new_stories = Story.mylist(@author, 1, 5)
          @fresh_stories = StorySheet.mylist(@author, 1, 5).map {|ss| ss.story}
        end
      }
      format.json { render json: @sheet.to_json(Sheet.show_json_opt) }
      format.atom 
      format.rss 
    end
  end

  def play
    @sheet = Sheet.show(params[:id], [@user, @admin])
    respond_to do |format|
      format.html {
        if @author
          @new_panels = Panel.mylist(@author, 1, 5)
        end
      }
      format.json {render json: @sheet.to_json(Sheet.show_json_opt)}
      format.jsonp {
        render :json => "callback(" + @sheet_panels.to_json_list + ");"
      }
    end
  end
  
  def count
    @sheet = {:count => Sheet.visible_count}
    respond_to do |format|
      format.json { render json: @sheet.to_json }
    end
  end
  
  def list
    @sheets = Sheet.all

    respond_to do |format|
      format.html { render layout: 'system' }# index.html.erb
      format.json { render json: @sheets }
    end
  end

  def browse
    @sheet = Sheet.find(params[:id])

    respond_to do |format|
      format.html { render layout: 'system' } # show.html.erb
      format.json { render json: @sheet }
    end
  end
  
  def new
    @sheet = Sheet.new
    @sheet.supply_default
    respond_to do |format|
      format.html # new.html.erb
      format.js
      format.json { render json: @sheet.to_json(Sheet.show_json_opt) }
    end
  end

  def edit
    @sheet = Sheet.edit(params[:id], @author)
    respond_to do |format|
      format.html 
      format.js
    end
  end

  def create
    @sheet = Sheet.new
    @sheet.supply_default 
    jsn = nil
    if params[:json]
      jsn = JSON.parse_no_except(params[:json])
    end
    @prm = params[:sheet] || jsn

    respond_to do |format|
      if @sheet.store @prm, @author
        flash[:notice] = I18n.t('flash.notice.created', :model => Sheet.model_name.human)
        format.html { redirect_to @sheet }
        format.json { render json: @sheet.to_json(Sheet.show_json_opt), status: :created, location: @sheet }
      else
        flash[:notice] = I18n.t('flash.notice.not_created', :model => Sheet.model_name.human)
        format.html { render action: "new" }
        format.json { render json: @sheet.errors, status: :unprocessable_entity }
      end
    end
  end

  def update
    @sheet = Sheet.edit(params[:id], @author)
    jsn = nil
    if params[:json]
      jsn = JSON.parse(params[:json])
    end
    @prm = params[:sheet] || jsn
    respond_to do |format|
      if @sheet.store @prm, @author
        flash[:notice] = I18n.t('flash.notice.updated', :model => Sheet.model_name.human)
        format.html { redirect_to @sheet }
        format.json { head :ok }
      else
        flash[:notice] = I18n.t('flash.notice.not_updated', :model => Sheet.model_name.human)
        format.html { render action: "edit" }
        format.json { render json: @sheet.errors, status: :unprocessable_entity }
      end
    end
  end

  def destroy
    @sheet = Sheet.edit(params[:id], @author)
    respond_to do |format|
      if @sheet.destroy_with_sheet_panel
        flash[:notice] = I18n.t('flash.notice.destroyed', :model => Sheet.model_name.human)
        format.html { redirect_to '/home/sheets' }
        format.json { head :ok }
      else
        flash[:notice] = I18n.t('flash.notice.not_destroyed', :model => Sheet.model_name.human)
        format.html { redirect_to @sheet }
        format.json { render json: @sheet.errors, status: :unprocessable_entity }
      end
    end
  end
end
