class ScrollPanel < ActiveRecord::Base
  belongs_to :author
  belongs_to :panel
  belongs_to :scroll
  
  validates :scroll_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :panel_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :author_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :t, :presence => true, :numericality => {:greater_than_or_equal_to => 0}
  
  def supply_default
    self.scroll_id = nil
    self.panel_id = nil
    self.t = nil
  end
  
  def overwrite au
    return false unless au
    self.author_id = au.id
  end
  
  def own? roles
    roles = [roles] unless roles.respond_to?(:each)
    au = ScrollPanel.get_author_from_roles roles
    return false unless au
    self.author_id == au.id
  end
  
  def visible? roles
    if MagicNumber['run_mode'] == 0
      return false unless guest_role_check(roles)
    else
      return false unless reader_role_check(roles)
    end
    return true if self.scroll.own?(roles)
    self.scroll.visible? roles
  end
  
  def self.default_panel_size
    30
  end
  
  def self.max_panel_size
    200
  end
  
  def self.offset cnt, prm = nil
    offset = prm.to_i
    offset = cnt - 1 if offset >= cnt
    offset = cnt - offset.abs if offset < 0
    offset = 0 if offset < 0
    offset
  end
  
  def self.panel_count cnt, prm = self.default_panel_size
    count = prm.to_i
    count = self.max_panel_size if count > self.max_panel_size
    count = self.default_panel_size if count < 1
    count
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.play_list_where cid
    ['scroll_panels.scroll_id = ?', cid]
  end
  
  def self.list_where
    'scrolls.visible > 0'
  end
  
  def self.mylist_where au
    ['scroll_panels.author_id = ?', au.id]
  end
  
  def self.himlist_where au
    ['scroll_panels.author_id = ? and scrolls.visible > 0', au.id]
  end
  
  def self.play_list scroll, author, offset = 0, limit = ScrollPanel.default_panel_size
    ScrollPanel.where(self.play_list_where(scroll.id)).includes(ScrollPanel.list_opt).order('scroll_panels.t').offset(offset).limit(limit)
  end
  
  def self.list page = 1, page_size = self.default_page_size
    ScrollPanel.where(self.list_where()).includes(ScrollPanel.list_opt).order('scroll_panels.updated_at desc').offset((page -1) * page_size).limit(page_size)
  end
  
  def self.mylist au, page = 1, page_size = Author.default_scroll_panel_page_size
    ScrollPanel.where(self.mylist_where(au)).includes(ScrollPanel.list_opt).order('scroll_panels.updated_at desc').offset((page -1) * page_size).limit(page_size)
  end
  
  def self.himlist au, page = 1, page_size = Author.default_scroll_panel_page_size
    ScrollPanel.where(self.himlist_where(au)).includes(ScrollPanel.list_opt).order('scroll_panels.updated_at desc').offset((page -1) * page_size).limit(page_size)
  end
  
  def self.list_paginate page = 1, page_size = self.default_page_size
    Kaminari.paginate_array(Array.new(ScrollPanel.where(self.list_where()).includes(ScrollPanel.list_opt).count, nil)).page(page).per(page_size)
  end
  
  def self.mylist_paginate au, page = 1, page_size = Author.default_scroll_panel_page_size
    Kaminari.paginate_array(Array.new(ScrollPanel.where(self.mylist_where(au)).includes(ScrollPanel.list_opt).count, nil)).page(page).per(page_size)
  end
  
  def self.himlist_paginate au, page = 1, page_size = Author.default_scroll_panel_page_size
    Kaminari.paginate_array(Array.new(ScrollPanel.where(self.himlist_where(au)).includes(ScrollPanel.list_opt).count, nil)).page(page).per(page_size)
  end
  
  def self.list_opt
    {
      :author => {}, 
      :scroll => {
        :author => {}
      }, 
      :panel => {
        :author => {}, 
        :panel_pictures => {:picture => {:artist => {}, :license => {}}}, 
        :speech_balloons =>{:balloon => {}, :speech => {}}
      }
    }
  end
  
  def self.list_json_opt
    {:include => {
      :author => {}, 
      :scroll => {
        :author => {}
      }, 
      :panel => {
        :author => {}, 
        :panel_pictures => {:picture => {:artist => {}, :license => {}}}, 
        :speech_balloons =>{:balloon => {}, :speech => {}}
      }
    }}
  end
  
  def self.show spid, roles
    opt = {}
    opt.merge!(ScrollPanel.show_opt)
    res = ScrollPanel.find spid, opt
    raise ActiveRecord::Forbidden unless res.visible?(roles)
    res
  end
  
  def self.edit spid, au
    opt = {}
    opt.merge!(ScrollPanel.show_opt)
    res = ScrollPanel.find spid, opt
    raise ActiveRecord::Forbidden unless res.own?(au)
    res
  end
  
  def self.show_opt
    {:include => {
      :author => {}, 
      :scroll => {
        :author => {}
      }, 
      :panel => {
        :author => {}, 
        :panel_pictures => {:picture => {:artist => {}, :license => {}}}, 
        :speech_balloons =>{:balloon => {}, :speech => {}}
      }
    }}
  end
  
  def elements
    self.panel.elements
  end
  
  def scroll_panel_as_json au
    panel_include = if self.panel and self.panel.visible?(au)
      {:include => {:author => {}}, :methods => :elements}
    else
      {:include => {:author => {}}}
    end
    self.to_json({:include => {:scroll => {:include => {:author => {}}}, :author => {}, :panel => panel_include}})
  end
  
  def self.list_as_json_text ary, au
    '[' + ary.map {|i| i.scroll_panel_as_json(au) }.join(',') + ']'
  end
  
  def self.licensed_pictures scroll_panels
    r = {}
    scroll_panels.each do |scroll_panel|
      r.merge!(scroll_panel.panel.licensed_pictures) if scroll_panel.panel
    end
    r
  end
  
  def self.new_t scroll_id
    r = ScrollPanel.max_t(scroll_id)
    r.blank? ? 0 : r.to_i + 1
  end
  
  def self.max_t scroll_id
    ScrollPanel.maximum(:t, :conditions => ['scroll_id = ?', scroll_id])
  end
  
  def self.find_t scroll_id, t
    ScrollPanel.find(:first, :conditions => ['scroll_id = ? and t = ?', scroll_id, t])
  end
  
  def self.collect_t scroll_panel
    r = ScrollPanel.find(:all, :conditions => ['scroll_id = ?', scroll_panel.scroll_id], :order => 't')
    r.map {|sp| sp.t}
  end
  
  def self.serial? ary
    i = 0
    ary.compact.sort.each do |t|
      break false unless t == i
      i += 1
    end
    ary.compact.size == i
  end
  
  def self.validate_t scroll_panel
    ScrollPanel.serial?(ScrollPanel.collect_t(scroll_panel))
  end
  
  def insert_shift
    ScrollPanel.update_all('t = t + 1', ['scroll_id = ? and t >= ?', self.scroll_id, self.t])
  end
  
  def lesser_shift old_t
    self.t = 0 if self.t < 0
    ScrollPanel.update_all('t = t + 1', ['scroll_id = ? and (t >= ? and t < ?)', self.scroll_id, self.t, old_t])
  end
  
  def higher_shift old_t
    nf = ScrollPanel.find_t(self.scroll_id, self.t)
    max_t = ScrollPanel.max_t(self.scroll_id).to_i
    self.t = max_t if self.t > max_t
    ScrollPanel.update_all('t = t - 1', ['scroll_id = ? and (t > ? and t <= ?)', self.scroll_id, old_t, self.t])
  end
  
  def update_shift old_t
    if self.t > old_t
      higher_shift old_t
    else
      lesser_shift old_t
    end
  end
  
  def rotate old_t = nil
    if self.new_record?
      if self.t.blank?
        self.t = ScrollPanel.new_t self.scroll_id
      else
        self.insert_shift
      end
    else
      if self.t.blank?
      else
        self.update_shift old_t
      end
    end
  end
  
  def allow?
    return nil if self.scroll_id == nil or self.panel_id == nil
    self.scroll.own?(self.author) and self.panel.usable?(self.author)
  end
  
  def store old_t = nil
    res = false
    ScrollPanel.transaction do
      case self.allow?
      when true
        self.rotate old_t
      when false
        raise ActiveRecord::Forbidden
      else
      end
      res = self.save
      raise ActiveRecord::Rollback unless res
      res = ScrollPanel.validate_t(self) 
      unless res
        self.errors.add :t, 'unserialized'
        raise ActiveRecord::Rollback 
      end
    end
    res
  end
  
  def destroy_and_shorten
    res = false
    ScrollPanel.transaction do
      ScrollPanel.update_all('t = t - 1', ['scroll_id = ? and (t > ?)', self.scroll_id, self.t])
      raise ActiveRecord::Rollback unless self.destroy
      res = true
    end
    res
  end
  
end
